#!/usr/bin/env python
# -*-coding:utf-8-*-
"""
Author : ming
date   : 2018年1月12日13:43:27
role   : 定制 Application
"""

from shortuuid import uuid
from tornado import httpserver, ioloop
from tornado import options as tnd_options
from tornado.options import options, define
from tornado.web import Application as tornadoApp
from .web_logs import ins_log

from .configs import configs

define("addr", default='0.0.0.0', help="run on the given ip address", type=str)
define("port", default=8000, help="run on the given port", type=int)
define("progid", default=str(uuid()), help="tornado progress id", type=str)


class Application(tornadoApp):
    """ 定制 Tornado Application 集成日志、sqlalchemy 等功能 """

    def __init__(self, handlers=None, default_host="", transforms=None, **settings):
        tnd_options.parse_command_line()
        if configs.can_import:
            configs.import_dict(**settings)
        ins_log.read_log('info', '%s' % options.progid)
        super(Application, self).__init__(handlers, default_host, transforms, **configs)
        http_server = httpserver.HTTPServer(self)
        http_server.listen(options.port, address=options.addr)
        self.io_loop = ioloop.IOLoop.instance()

    def start_server(self):
        """
        启动 tornado 服务
        :return:
        """
        try:
            ins_log.read_log('info', 'progressid: %(progid)s' % dict(progid=options.progid))
            ins_log.read_log('info', 'server address: %(addr)s:%(port)d' % dict(addr=options.addr, port=options.port))
            ins_log.read_log('info', 'web server start sucessfuled.')
            self.io_loop.start()
        except KeyboardInterrupt:
            self.io_loop.stop()
        except:
            import traceback
            ins_log.read_log('error', '%(tra)s' % dict(tra=traceback.format_exc()))


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*-coding:utf-8-*-
'''
author : shenshuo
date   : 2018年2月5日13:37:54
role   : 运维操作
'''

import os
import sys
import time
import re
import subprocess
import base64
import rsa
from Crypto.Cipher import AES
from binascii import b2a_hex, a2b_hex


###执行shell命令函数
def exec_shell(cmd):
    sub2 = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    stdout, stderr = sub2.communicate()
    ret = sub2.returncode
    if ret == 0:
        return ret, stdout.decode('utf-8').split('\n')
    else:
        return ret, stdout.decode('utf-8').replace('\n', '')


###脚本排它函数
def exclusiveLock(scriptName):
    pid_file = '/tmp/%s.pid' % scriptName
    lockcount = 0
    while True:
        if os.path.isfile(pid_file):
            ###打开脚本运行进程id文件并读取进程id
            fp_pid = open(pid_file, 'r')
            process_id = fp_pid.readlines()
            fp_pid.close()

            ###判断pid文件取出的是否是数字
            if not process_id:
                break

            if not re.search(r'^\d', process_id[0]):
                break

            ###确认此进程id是否还有进程
            lockcount += 1
            if lockcount > 4:
                print('2 min after this script is still exists')
                sys.exit(111)
            else:
                if os.popen('/bin/ps %s|grep "%s"' % (process_id[0], scriptName)).readlines():
                    print("The script is running...... ,Please wait for a moment!")
                    time.sleep(30)
                else:
                    os.remove(pid_file)
        else:
            break

    ###把进程号写入文件
    wp_pid = open(pid_file, 'w')
    sc_pid = os.getpid()
    wp_pid.write('%s' % sc_pid)
    wp_pid.close()


### 加密解密模块
class MyCrypt:
    """
    usage: mc = MyCrypt()               实例化
        mc.my_encrypt('ceshi')          对字符串ceshi进行加密
        mc.my_decrypt('')               对密文进行解密
    """

    def __init__(self, key='HOrUmuJ4bCVG6EYu2docoRNNYSdDpJJw'):
        # 这里密钥key 长度必须为16（AES-128）、24（AES-192）、或32（AES-256）Bytes 长度
        self.key = key
        self.mode = AES.MODE_CBC

    def my_encrypt(self, text):
        length = 32
        count = len(text)
        if count < length:
            add = length - count
            text = text + ('\0' * add)

        elif count > length:
            add = (length - (count % length))
            text = text + ('\0' * add)

        cryptor = AES.new(self.key, self.mode, b'0000000000000000')
        self.ciphertext = cryptor.encrypt(text)
        return b2a_hex(self.ciphertext).decode('utf-8')

    def my_decrypt(self, text):
        cryptor = AES.new(self.key, self.mode, b'0000000000000000')
        plain_text = cryptor.decrypt(a2b_hex(text)).decode('utf-8')
        return plain_text.rstrip('\0')


class MyCryptV2:

    def __init__(self, key='HOrUmuJ4bCVG6EYu2docoRNNYSdDpJJw'):
        """
        Usage:
            #实例化
            mc = MyCrypt()
            #加密方法
            mc.my_encrypt('password')
            #解密方法
            mc.my_decrypt('ZpZjEcsqnySTz6UsXD/+TA==')
        :param key:
        """
        self.key = key

    # str不是16的倍数那就补足为16的倍数
    def add_to_16(self, value):
        while len(value) % 16 != 0:
            value += '\0'
        return str.encode(value)  # 返回bytes

    def my_encrypt(self, text):
        """
        加密方法
        :param text: 密码
        :return:
        """
        aes = AES.new(self.add_to_16(self.key), AES.MODE_ECB)
        # 先进行aes加密

        encrypt_aes = aes.encrypt(self.add_to_16(text))
        # 用base64转成字符串形式
        encrypted_text = str(base64.encodebytes(encrypt_aes), encoding='utf-8').replace('\n', '')  # 执行加密并转码返回bytes
        # print('[INFO]: 你的加密为：{}'.format(encrypted_text))
        return encrypted_text

    def my_decrypt(self, text):
        """
        解密方法
        :param text: 加密后的密文
        :return:
        """
        # 初始化加密器
        aes = AES.new(self.add_to_16(self.key), AES.MODE_ECB)
        # 优先逆向解密base64成bytes
        base64_decrypted = base64.decodebytes(text.encode(encoding='utf-8'))
        # 执行解密密并转码返回str
        decrypted_text = str(aes.decrypt(base64_decrypted), encoding='utf-8').replace('\0', '')
        # print('[INFO]: 你的解密为：{}'.format(decrypted_text))
        return decrypted_text


class MyCryptV3:
    """
    加密：公钥加密，私钥解密；
    签名：私钥签名，公钥验签。
    用作加密时，密文泄露是无所谓的（相对而言），重要的是用于解密的密钥必须安全，所以用不公开的私钥来解密，用公钥来加密；
    用作签名时，目的是防止别人伪造我的身份发信息，所以用私钥来签名，用公钥来验签。
    """

    @classmethod
    def check_message(cls, message):
        if isinstance(message, str):  # 如果msg为字符串, 则转化为字节类型
            message = message.encode('utf-8')
        elif isinstance(message, bytes):  # 如果msg为字节类型, 则无需处理
            pass
        else:  # 否则抛出异常
            raise TypeError('message必须为字符串或者字节类型!')
        return message

    @classmethod
    def create_new_keys(cls, num=4096):  # 可以加密数据长度 4096/8 - 11
        public_key, private_key = rsa.newkeys(num)

        with open('public.pem', 'w+') as f:
            f.write(public_key.save_pkcs1().decode())
        with open('private.pem', 'w+') as f:
            f.write(private_key.save_pkcs1().decode())
        return public_key, private_key

    @classmethod
    def load_private_key(cls, private_path=None, private_info=None):
        if not private_path and not private_info:  raise ValueError('两个参数不能都为空')

        if private_path:
            with open(private_path, "rb") as private_file:
                p = private_file.read()
                return rsa.PrivateKey.load_pkcs1(p)
        else:
            if isinstance(private_info, str):  # 如果pub_key为字符串, 则转化为字节类型
                private_info = private_info.encode('utf-8')
            elif isinstance(private_info, bytes):  # 如果key为字节类型, 则无需处理
                pass
            else:  # 否则抛出异常
                raise TypeError('私钥必须为None、字符串或者字节类型!')
            return rsa.PrivateKey.load_pkcs1(private_info)

    @classmethod
    def load_public_key(cls, public_path=None, public_info=None):
        if not public_path and not public_info:  raise ValueError('两个参数不能都为空')

        if public_path:
            with open(public_path, "rb") as public_file:
                p = public_file.read()
                return rsa.PublicKey.load_pkcs1(p)
        else:
            if isinstance(public_info, str):  # 如果pub_key为字符串, 则转化为字节类型
                public_info = public_info.encode('utf-8')
            elif isinstance(public_info, bytes):  # 如果key为字节类型, 则无需处理
                pass
            else:  # 否则抛出异常
                raise TypeError('公钥必须为None、字符串或者字节类型!')

            return rsa.PublicKey.load_pkcs1(public_info)

    @classmethod
    def to_sign_with_private_key(cls, message, private_path=None, private_key=None):
        # 私钥签名
        message = cls.check_message(message)
        private_key_obj = cls.load_private_key(private_path, private_key)
        result = rsa.sign(message, private_key_obj, 'SHA-384')
        return result

    @classmethod
    def to_verify_with_public_key(cls, message, signature, public_path=None, public_key=None):
        # 公钥验签
        public_key_obj = cls.load_public_key(public_path, public_key)
        message = cls.check_message(message)
        result = rsa.verify(message, signature, public_key_obj)
        return result

    @classmethod
    def to_decrypt(cls, message, private_path=None, private_key=None, be_base64=True):
        # 非对称解密
        if be_base64: message = base64.b64decode(message)
        message = cls.check_message(message)

        private_key_obj = cls.load_private_key(private_path, private_key)

        return rsa.decrypt(message, private_key_obj).decode()

    @classmethod
    def to_encrypt(cls, message, public_path=None, public_key=None, be_base64=True):
        # 非对称加密
        """
        :param message: 待加密字符串或者字节
        :param public_path: 公钥路径  二选一
        :param public_key: 公钥字符串  二选一
        :return: base64密文字符串
        """

        message = cls.check_message(message)
        public_key_obj = cls.load_public_key(public_path, public_key)

        cryto_msg = rsa.encrypt(message, public_key_obj)  # 生成加密文本
        if be_base64: return base64.b64encode(cryto_msg).decode("utf-8")  # 将加密文本转化为 base64 编码

        return cryto_msg  # 将字节类型的 base64 编码转化为字符串类型


### 当前时间
def now_time():
    return time.strftime('%Y-%m-%d-%H-%M-%S', time.localtime(time.time()))


def is_ip(ip):
    if re.search(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b$', ip):
        return True
    return False


if __name__ == "__main__":
    # 待加密字符串或者字节
    ceshiyixia = "富强、民主、文明、和谐”，是我国社会主义现代化国家的建设目标，也是从价值目标层面对社会主义核心价值观基本理念的凝练，在社会主义核心价值观中居于最高层次，对其他层次的价值理念具有统领作用。"

    # MyCryptV3.create_new_keys(num=4096)  ## 4096/8 - 11 生成密钥对

    ## 公钥加密
    cryto_info = MyCryptV3.to_encrypt(ceshiyixia, public_path='public.pem', public_key=None)
    print(cryto_info)

    ## 私钥解密
    haha_info = MyCryptV3.to_decrypt(cryto_info, private_path='private.pem', private_key=None)
    print(haha_info)

    ## 签名
    eee_info = MyCryptV3.to_sign_with_private_key(ceshiyixia, private_path='private.pem', private_key=None)
    print(eee_info)

    ## 验签
    aaa_info = MyCryptV3.to_verify_with_public_key(ceshiyixia, eee_info, public_path='public.pem')
    print(aaa_info)

    public_key = ''
    private_key = ''
    ## 公钥加密
    cryto_info = MyCryptV3.to_encrypt(ceshiyixia, public_path=None, public_key=public_key)
    print(cryto_info)

    ## 私钥解密
    haha_info = MyCryptV3.to_decrypt(cryto_info, private_path=None, private_key=private_key)
    print(haha_info)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2018年2月5日13:37:54
Desc    ：记录API
"""


class TaskAPIS:
    create_jobs= dict(method='POST',
                     url='/task/v2/task/accept/',
                     body={
                         "task_name": "任务名称",
                         "submitter": "提交人",
                         "temp_id": "200",
                         "schedule": "ready",
                         "exec_time": "2018-11-27 14:09:50",
                         "associated_user": "{'group-1': ['奥特曼']}",
                         "args": "{'VERSION':'eeee', 'arg02': 'xxxx'}",
                         "details": "这里是备注",
                         "hosts": "{1: '127.0.0.1'}"
                     },
                     field_help={
                         "task_name": "任务名称",
                         "submitter": "提交人",
                         "temp_id": "模板ID，就是你创建模板时候生成的那个ID",
                         "schedule": "这是状态，常用的有ready和new ready：表示不通过人工审核，只要到了执行时间直接执行任务\
                         new：表示需要任何审核，管理员审核，选择执行时间，到时间后开始执行",
                         "exec_time": "任务执行时间，状态为ready的情况下，到这个时间会进行执行",
                         "associated_user": "{'group-1': ['奥特曼']}",
                         "args": "这里是一个字典，里面的参数可以自行定义，如上，你模板参数里面用到了哪些你都可以在这里定义出来，当你的POST到这个接口时候，我们会自动接受此参数，并帮你运行脚本 解析你要传入的参数。",
                         "details": "描述，备注信息",
                         "hosts": "这个是执行主机，字典形式， 1表示第一组主机，也就是上面模板里面的组1，"
                                  "任务支持多组。主机IP，这个是执行主机，这个废话多一点，比如我以上模板的脚本在172.16.0.101这台主机上，"
                                  "我就想平台登陆我这个主机，来帮我执行这些脚本，至于怎么登陆，那么就是我最开始在平台里面配置了一个执行用户，"
                                  "我将我这个主机的私钥放到了平台上，公钥在我服务器上，这样子就可以ssh -i xxxx.pem@{ip_address}\
                                  远程到我的主机上帮我执行命令。"
                     },
                     description='基于此接口可以创建作业任务')



#!/usr/bin/env python
# -*-coding:utf-8-*-
"""
Author : shenshuo
Date   : 2019年12月11日
Desc   : models类
"""

from datetime import datetime
from sqlalchemy.orm import class_mapper
from .utils import get_contain_dict
from .db_context import DBContextV2 as DBContext
from sqlalchemy import text


def model_to_dict(model):
    model_dict = {}
    for key, column in class_mapper(model.__class__).c.items():
        if isinstance(getattr(model, key), datetime):
            model_dict[column.name] = str(getattr(model, key))
        else:
            model_dict[column.name] = getattr(model, key, None)
    return model_dict


def queryset_to_list(queryset, **kwargs) -> list:
    if kwargs: return [model_to_dict(q) for q in queryset if get_contain_dict(kwargs, model_to_dict(q))]
    return [model_to_dict(q) for q in queryset]


def GetInsertOrUpdateObj(cls: classmethod, str_filter: str, **kw) -> classmethod:
    """
    cls:            Model 类名
    str_filter:      filter的参数.eg:"name='name-14'" 必须设置唯一 支持 and or
    **kw:           【属性、值】字典,用于构建新实例，或修改存在的记录
    session.add(GetInsertOrUpdateObj(TableTest, "name='name-114'", age=33114, height=123.14, name='name-114'))
    """
    with DBContext('r') as session:
        existing = session.query(cls).filter(text(str_filter)).first()
    if not existing:
        res = cls()
        for k, v in kw.items():
            if hasattr(res, k):
                setattr(res, k, v)
        return res
    else:
        res = existing
        for k, v in kw.items():
            if hasattr(res, k):
                setattr(res, k, v)

        return res



from enum import IntEnum
from .consts import ErrorCode
class BaseError(Exception):
    """ 错误基类，所有错误必须从该类继承 """

    def __init__(self, errorcode, *args, **kwargs):
        """
        初始化错误基类
        :param errorcode: 错误码
        :param args:
        :param kwargs:
        """
        if isinstance(errorcode, IntEnum):
            self._errorcode = errorcode
            self.kwargs = kwargs
            super(BaseError, self).__init__(*args)
        else:
            raise TypeError(
                'Error code must be vmbsdk.constants.enums.ErrorCode type.')

    @property
    def errorcode(self):
        return self._errorcode
class BizError(BaseError):
    """ 业务错误 """

    def __init__(self, errorcode, *args, **kwargs):
        self.__subcode = int(args[1]) if len(args) > 1 else 0
        super(BizError, self).__init__(errorcode, *args, **kwargs)

    @property
    def subcode(self):
        """
        获取
        :return:
        """
        return self.__subcode

class BadRequestError(BizError):
    """ 错误的请求 """

    def __init__(self, *args, **kwargs):
        super(BadRequestError, self).__init__(ErrorCode.bad_request,
                                              *args, **kwargs)


class ConfigError(Exception):
    def __init__(self, config_key, *args, **kwargs):
        self.config_key = config_key
        super(ConfigError, self).__init__(*args, **kwargs)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Contact : 191715030@qq.com
Author  : shenshuo
Date    : 2019/4/17
Desc    : 对接LDAP登录认证
"""

from ldap3 import Server, Connection, ALL, SUBTREE, ServerPool


class LdapApi:
    def __init__(self, ldap_server_host, ldap_admin_dn, ldap_admin_password, ldap_server_port=389, ldap_use_ssl=False):
        self._ldap_admin_dn = ldap_admin_dn
        self._ldap_admin_password = ldap_admin_password
        # ldap_server_pool = ServerPool(["172.16.0.102",'172.16.0.103'])
        self.ldap_server = Server(ldap_server_host, port=ldap_server_port, use_ssl=ldap_use_ssl)

    def ldap_server_test(self):
        try:
            conn = Connection(self.ldap_server, user=self._ldap_admin_dn, password=self._ldap_admin_password,
                              check_names=True, lazy=False, raise_exceptions=False)
            conn.open()
            conn.bind()
            return True
        except Exception as e:
            print("auth fail {}".format(e))
            return False

    def ldap_auth(self, username, password, search_base, search_filter='cn'):
        if not self.ldap_server_test():
            return False, None, None

        conn = Connection(self.ldap_server, user=self._ldap_admin_dn, password=self._ldap_admin_password,
                          check_names=True, lazy=False, raise_exceptions=False)
        conn.open()
        conn.bind()

        res = conn.search(search_base=search_base,
                          search_filter='({}={})'.format(search_filter, username),
                          search_scope=SUBTREE,
                          # attributes=['cn', 'givenName', 'email', 'mail', 'sAMAccountName'],
                          attributes=['cn', 'email', 'mail'], paged_size=5)

        if res:
            entry = conn.response[0]
            dn = entry['dn']
            attr_dict = entry['attributes']

            # check password by dn
            try:
                conn2 = Connection(self.ldap_server, user=dn, password=password, check_names=True, lazy=False,
                                   raise_exceptions=False)
                conn2.bind()
                if conn2.result["description"] == "success":
                    if 'email' in attr_dict:
                        email = attr_dict["email"][0] if isinstance(attr_dict["email"], list) else attr_dict["email"]
                    elif 'mail' in attr_dict:
                        email = attr_dict["mail"][0] if isinstance(attr_dict["mail"], list) else attr_dict["mail"]
                    else:
                        email = None

                    return True, attr_dict["cn"][0], email
                else:
                    print("auth fail")
                    return False, None, None
            except Exception as e:
                print("auth fail {}".format(e))
                return False, None, None
        else:
            return False, None, None

    def ldap_auth_v2(self, username, password, search_base, search_filter='cn'):
        if not self.ldap_server_test(): return False, None, None

        conn = Connection(self.ldap_server, user=self._ldap_admin_dn, password=self._ldap_admin_password,
                          check_names=True, lazy=False, raise_exceptions=False)
        conn.open()
        conn.bind()

        res = conn.search(search_base=search_base, search_filter=f'({search_filter}={username})',
                          search_scope=SUBTREE, attributes=[search_filter, 'email', 'mail'], paged_size=5)

        if not res: return False, None, None

        entry = conn.response[0]
        dn = entry['dn']
        attr_dict = entry['attributes']

        # check password by dn
        try:
            conn2 = Connection(self.ldap_server, user=dn, password=password, check_names=True, lazy=False,
                               raise_exceptions=False)
            conn2.bind()
            if conn2.result["description"] == "success":
                try:
                    if 'email' in attr_dict:
                        email = attr_dict["email"][0] if isinstance(attr_dict["email"], list) else attr_dict["email"]
                    elif 'mail' in attr_dict:
                        email = attr_dict["mail"][0] if isinstance(attr_dict["mail"], list) else attr_dict["mail"]
                    else:
                        email = None
                except Exception as err:
                    print(f"email fail, {err}")
                    email = None

                return True, attr_dict[search_filter][0], email
            else:
                print("auth fail")
                return False, None, None
        except Exception as e:
            print("auth fail {}".format(e))
            return False, None, None


if __name__ == "__main__":
    obj = LdapApi('172.16.0.102', 'cn=Manager,DC=sz,DC=com', '070068')
    print(obj.ldap_server_test())
    print('____________')
    print(obj.ldap_auth("yanghongfei", "123456", 'ou=opendevops,dc=sz,dc=com', 'cn'))



#!/usr/bin/env python
# -*-coding:utf-8-*-
'''
Author : SS
date   : 2017年12月29日14:43:24
role   : 集中化管理工具的使用
'''

import requests
import json
import time

try:
    import cookielib
except:
    import http.cookiejar as cookielib

import ssl

context = ssl._create_unverified_context()
import urllib3

urllib3.disable_warnings()


class SaltApi:
    """
    定义salt api接口的类
    初始化获得token
    """

    def __init__(self, url='https://127.0.0.1:8001/', username="saltapi", password="shenshuo"):
        self.__url = url
        self.__username = username
        self.__password = password
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36",
            "Content-type": "application/json"
            # "Content-type": "application/x-yaml"
        }
        self.params = {'client': 'local', 'fun': '', 'tgt': ''}
        self.login_url = self.__url + "login"
        self.login_params = {'username': self.__username, 'password': self.__password, 'eauth': 'pam'}
        self.token = self.get_data(self.login_url, self.login_params)['token']
        self.headers['X-Auth-Token'] = self.token

    def get_data(self, url, params):
        send_data = json.dumps(params)
        request = requests.post(url, data=send_data, headers=self.headers, verify=False)
        response = request.json()
        result = dict(response)
        return result['return'][0]

    def salt_command(self, tgt, method, arg=None):
        """远程执行命令，相当于salt 'client1' cmd.run 'free -m'"""
        if arg:
            params = {'client': 'local', 'fun': method, 'tgt': tgt, 'arg': arg}
        else:
            params = {'client': 'local', 'fun': method, 'tgt': tgt}
        result = self.get_data(self.__url, params)
        return result

    def salt_async_command(self, tgt, method, arg=None):  # 异步执行salt命令，根据jid查看执行结果

        """远程异步执行命令"""
        if arg:
            params = {'client': 'local_async', 'fun': method, 'tgt': tgt, 'arg': arg}
        else:
            params = {'client': 'local_async', 'fun': method, 'tgt': tgt}
        jid = self.get_data(self.__url, params).get('jid', None)
        return jid

    def look_jid(self, jid):  # 根据异步执行命令返回的jid查看事件结果
        params = {'client': 'runner', 'fun': 'jobs.lookup_jid', 'jid': jid}
        result = self.get_data(self.__url, params)
        return result

    def run(self, salt_client='*', salt_method='cmd.run_all', salt_params='w', timeout=1800):
        try:
            if not self.salt_command(salt_client, 'test.ping')[salt_client]:
                return -98, 'test.ping error 98', ''
        except Exception as e:
            return -99, 'test.ping error', str(e)

        t = 0
        jid = self.salt_async_command(salt_client, salt_method, salt_params)
        if not jid:
            return -100, '连接失败', '连接失败或主机不存在'

        while True:
            time.sleep(5)
            if t == timeout:
                print('exec timeout!')
                break
            else:
                t += 5
            result = self.look_jid(jid)
            for i in result.keys():
                return result[i]['retcode'], result[i]['stdout'], result[i]['stderr']


if __name__ == '__main__':
    pass
